@@ -29,9 +29,9 @@ from abc import ABC, abstractmethod
 from enum import Enum
 from typing import Any, Dict, List, Optional, Tuple, Union
 
+import bcj  # type: ignore  # noqa
 import pyppmd
 import pyzstd
-from zipfile_deflate64 import deflate64  # type: ignore
 from Cryptodome.Cipher import AES
 from Cryptodome.Random import get_random_bytes
 
@@ -61,14 +61,14 @@ from py7zr.properties import (
     get_default_blocksize,
 )
 
-try:
-    import bcj as BCJFilter  # type: ignore  # noqa
-except ImportError:
-    import py7zr.bcjfilter as BCJFilter  # type: ignore  # noqa
 try:
     import brotli  # type: ignore  # noqa
 except ImportError:
     import brotlicffi as brotli  # type: ignore  # noqa
+try:
+    import inflate64 # noqa
+except ImportError:
+    inflate64 = None  # type: ignore # noqa
 brotli_major = 1
 brotli_minor = 0
 
@@ -265,22 +265,46 @@ class DeflateDecompressor(ISevenZipDecompressor):
 
 class Deflate64Compressor(ISevenZipCompressor):
     def __init__(self):
-        raise RuntimeError("Deflate64 compression not implemented yet")
+        self.flushed = False
+        if inflate64 is not None:
+            self._compressor = inflate64.Deflater()
+            self._enabled = True
+        else:
+            self._enabled = False
+
+    def compress(self, data: Union[bytes, bytearray, memoryview], max_length: int = -1) -> bytes:
+        if not self._enabled:
+            raise UnsupportedCompressionMethodError(None, "inflate64 is not installed.")
+        return self._compressor.deflate(data)
+
+    def flush(self) -> bytes:
+        if not self._enabled:
+            raise UnsupportedCompressionMethodError(None, "inflate64 is not installed.")
+        if self.flushed:
+            return b""
+        self.flushed = True
+        return self._compressor.flush()
 
 
 class Deflate64Decompressor(ISevenZipDecompressor):
     def __init__(self):
         self.flushed = False
-        self._decompressor = deflate64.Deflate64()
+        if inflate64 is not None:
+            self._decompressor = inflate64.Inflater()
+            self._enabled = True
+        else:
+            self._enabled = False
 
     def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int = -1) -> bytes:
+        if not self._enabled:
+            raise UnsupportedCompressionMethodError(None, "inflate64 is not installed.")
         if len(data) == 0:
             if self.flushed:
                 return b""
             else:
                 self.flushed = True
-                return self._decompressor.flush()
-        return self._decompressor.decompress(data)
+                return self._decompressor.inflate(b"")
+        return self._decompressor.inflate(data)
 
 
 class CopyCompressor(ISevenZipCompressor):
@@ -358,7 +382,7 @@ class PpmdCompressor(ISevenZipCompressor):
 
 class BcjSparcDecoder(ISevenZipDecompressor):
     def __init__(self, size: int):
-        self.decoder = BCJFilter.SparcDecoder(size)
+        self.decoder = bcj.SparcDecoder(size)
 
     def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int = -1) -> bytes:
         return self.decoder.decode(data)
@@ -366,7 +390,7 @@ class BcjSparcDecoder(ISevenZipDecompressor):
 
 class BcjSparcEncoder(ISevenZipCompressor):
     def __init__(self):
-        self.encoder = BCJFilter.SparcEncoder()
+        self.encoder = bcj.SparcEncoder()
 
     def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes:
         return self.encoder.encode(data)
@@ -377,7 +401,7 @@ class BcjSparcEncoder(ISevenZipCompressor):
 
 class BcjPpcDecoder(ISevenZipDecompressor):
     def __init__(self, size: int):
-        self.decoder = BCJFilter.PPCDecoder(size)
+        self.decoder = bcj.PPCDecoder(size)
 
     def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int = -1) -> bytes:
         return self.decoder.decode(data)
@@ -385,7 +409,7 @@ class BcjPpcDecoder(ISevenZipDecompressor):
 
 class BcjPpcEncoder(ISevenZipCompressor):
     def __init__(self):
-        self.encoder = BCJFilter.PPCEncoder()
+        self.encoder = bcj.PPCEncoder()
 
     def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes:
         return self.encoder.encode(data)
@@ -396,7 +420,7 @@ class BcjPpcEncoder(ISevenZipCompressor):
 
 class BcjArmtDecoder(ISevenZipDecompressor):
     def __init__(self, size: int):
-        self.decoder = BCJFilter.ARMTDecoder(size)
+        self.decoder = bcj.ARMTDecoder(size)
 
     def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int = -1) -> bytes:
         return self.decoder.decode(data)
@@ -404,7 +428,7 @@ class BcjArmtDecoder(ISevenZipDecompressor):
 
 class BcjArmtEncoder(ISevenZipCompressor):
     def __init__(self):
-        self.encoder = BCJFilter.ARMTEncoder()
+        self.encoder = bcj.ARMTEncoder()
 
     def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes:
         return self.encoder.encode(data)
@@ -415,7 +439,7 @@ class BcjArmtEncoder(ISevenZipCompressor):
 
 class BcjArmDecoder(ISevenZipDecompressor):
     def __init__(self, size: int):
-        self.decoder = BCJFilter.ARMDecoder(size)
+        self.decoder = bcj.ARMDecoder(size)
 
     def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int = -1) -> bytes:
         return self.decoder.decode(data)
@@ -423,7 +447,7 @@ class BcjArmDecoder(ISevenZipDecompressor):
 
 class BcjArmEncoder(ISevenZipCompressor):
     def __init__(self):
-        self.encoder = BCJFilter.ARMEncoder()
+        self.encoder = bcj.ARMEncoder()
 
     def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes:
         return self.encoder.encode(data)
@@ -434,7 +458,7 @@ class BcjArmEncoder(ISevenZipCompressor):
 
 class BCJDecoder(ISevenZipDecompressor):
     def __init__(self, size: int):
-        self.decoder = BCJFilter.BCJDecoder(size)
+        self.decoder = bcj.BCJDecoder(size)
 
     def decompress(self, data: Union[bytes, bytearray, memoryview], max_length: int = -1) -> bytes:
         return self.decoder.decode(data)
@@ -442,7 +466,7 @@ class BCJDecoder(ISevenZipDecompressor):
 
 class BCJEncoder(ISevenZipCompressor):
     def __init__(self):
-        self.encoder = BCJFilter.BCJEncoder()
+        self.encoder = bcj.BCJEncoder()
 
     def compress(self, data: Union[bytes, bytearray, memoryview]) -> bytes:
         return self.encoder.encode(data)
@@ -628,10 +652,7 @@ class SevenZipDecompressor:
     def _decompress(self, data, max_length: int):
         for i, decompressor in enumerate(self.chain):
             if self._unpacked[i] < self._unpacksizes[i]:
-                if isinstance(decompressor, LZMA1Decompressor) or isinstance(decompressor, PpmdDecompressor):
-                    data = decompressor.decompress(data, max_length)  # always give max_length for lzma1
-                else:
-                    data = decompressor.decompress(data)
+                data = decompressor.decompress(data, max_length)
                 self._unpacked[i] += len(data)
             elif len(data) == 0:
                 data = b""
@@ -1139,12 +1160,6 @@ class SupportedMethods:
                 " and help Python core team implementing it."
                 " Or please use another tool to extract it.",
             )
-        if coder["method"] == COMPRESSION_METHOD.MISC_DEFLATE64:
-            raise UnsupportedCompressionMethodError(
-                coder["method"],
-                "DEFLATE64 compression is not supported by py7zr yet."
-                " Please check the progress in py7zr project home page.",
-            )
         if coder["method"] == COMPRESSION_METHOD.MISC_LZ4:
             raise UnsupportedCompressionMethodError(
                 coder["method"], "Archive is compressed by an unsupported algorythm LZ4."
